"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const bases = {
    ClassElement: {
        base: ['NamedDeclaration'],
        getChildren: (node) => [node.name],
    },
    ClassLikeDeclarationBase: {
        base: ['NamedDeclaration', 'JSDocContainer'],
        getChildren: (node) => [
            node.name,
            node.typeParameters,
            node.heritageClauses,
            node.members,
        ],
    },
    Declaration: {
        base: ['Node'],
    },
    DeclarationStatement: {
        base: ['NamedDeclaration', 'Statement'],
        getChildren: (node) => [node.name],
    },
    Expression: {
        base: ['Node'],
    },
    FunctionLikeDeclarationBase: {
        base: ['SignatureDeclarationBase'],
        getChildren: (node) => [
            node.asteriskToken,
            node.questionToken,
            node.body,
        ],
    },
    IterationStatement: {
        base: ['Statement'],
        getChildren: (node) => [node.statement],
    },
    JSDocContainer: {
        base: ['Node'],
    },
    JSDocPropertyLikeTag: {
        base: ['JSDocTag', 'Declaration'],
        getChildren: (node) => [
            node.name,
            node.typeExpression,
        ],
    },
    JSDocTag: {
        base: ['Node'],
        getChildren: (node) => [node.atToken, node.tagName],
    },
    JSDocType: {
        base: ['TypeNode'],
    },
    LeftHandSideExpression: {
        base: ['UpdateExpression'],
    },
    LiteralExpression: {
        base: ['LiteralLikeNode', 'PrimaryExpression'],
    },
    LiteralLikeNode: {
        base: ['Node'],
    },
    MemberExpression: {
        base: ['LeftHandSideExpression'],
    },
    NamedDeclaration: {
        base: ['Declaration'],
        getChildren: (node) => [node.name],
    },
    Node: {
        base: [],
    },
    ObjectLiteralElement: {
        base: ['NamedDeclaration'],
        getChildren: (node) => [node.name],
    },
    ObjectLiteralExpression: {
        base: ['ObjectLiteralExpressionBase'],
    },
    ObjectLiteralExpressionBase: {
        base: ['PrimaryExpression', 'Declaration'],
        getChildren: (node) => [
            node.properties,
        ],
    },
    PrimaryExpression: {
        base: ['MemberExpression'],
    },
    SignatureDeclarationBase: {
        base: ['NamedDeclaration', 'JSDocContainer'],
        getChildren: (node) => [
            node.name,
            node.typeParameters,
            node.parameters,
            node.type,
        ],
    },
    Statement: {
        base: ['Node'],
    },
    TypeElement: {
        base: ['NamedDeclaration'],
        getChildren: (node) => [node.name, node.questionToken],
    },
    TypeNode: {
        base: ['Node'],
    },
    UnaryExpression: {
        base: ['Expression'],
    },
    UpdateExpression: {
        base: ['UnaryExpression'],
    },
};
exports.default = bases;
//# sourceMappingURL=bases.js.map