"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const kinds = {
    AbstractKeyword: {
        base: ['Node'],
    },
    AmpersandAmpersandToken: {
        base: ['Node'],
        union: ['LogicalOperator'],
    },
    AmpersandEqualsToken: {
        base: ['Node'],
        union: ['CompoundAssignmentOperator'],
    },
    AmpersandToken: {
        base: ['Node'],
        union: ['BitwiseOperator'],
    },
    AnyKeyword: {
        base: ['TypeNode'],
        union: ['KeywordTypeNode'],
    },
    ArrayBindingPattern: {
        base: ['Node'],
        getChildren: (node) => [node.elements],
        union: ['ArrayBindingOrAssignmentPattern'],
    },
    ArrayLiteralExpression: {
        base: ['PrimaryExpression'],
        getChildren: (node) => [node.elements],
        union: [
            'BindingOrAssignmentElement',
            'ArrayBindingOrAssignmentPattern',
            'AssignmentPattern',
            'DestructuringPattern',
        ],
    },
    ArrayType: {
        base: ['TypeNode'],
        getChildren: (node) => [node.elementType],
    },
    ArrowFunction: {
        base: ['Expression'],
        getChildren: (node) => [
            node.equalsGreaterThanToken,
            node.body,
        ],
        union: ['HasJSDoc', 'SignatureDeclaration', 'FunctionLikeDeclaration'],
    },
    AsExpression: {
        base: ['Expression'],
        getChildren: (node) => [node.expression, node.type],
        union: ['AssertionExpression'],
    },
    AsKeyword: {
        base: ['Node'],
    },
    AsteriskAsteriskEqualsToken: {
        base: ['Node'],
        union: ['CompoundAssignmentOperator'],
    },
    AsteriskAsteriskToken: {
        base: ['Node'],
        union: ['ExponentiationOperator'],
    },
    AsteriskEqualsToken: {
        base: ['Node'],
        union: ['CompoundAssignmentOperator'],
    },
    AsteriskToken: {
        base: ['Node'],
        union: ['MultiplicativeOperator'],
    },
    AsyncKeyword: {
        base: ['Node'],
        union: ['Modifier'],
    },
    AtToken: {
        base: ['Node'],
    },
    AwaitExpression: {
        base: ['UnaryExpression'],
        getChildren: (node) => [node.expression],
    },
    AwaitKeyword: {
        base: ['Node'],
    },
    BarBarToken: {
        base: ['Node'],
        union: ['LogicalOperator'],
    },
    BarEqualsToken: {
        base: ['Node'],
        union: ['CompoundAssignmentOperator'],
    },
    BarToken: {
        base: ['Node'],
        union: ['BitwiseOperator'],
    },
    BinaryExpression: {
        base: ['Expression', 'Declaration'],
        getChildren: (node) => [
            node.left,
            node.operatorToken,
            node.right,
        ],
    },
    BindingElement: {
        base: ['NamedDeclaration'],
        getChildren: (node) => [
            node.propertyName,
            node.dotDotDotToken,
            node.name,
            node.initializer,
        ],
        union: ['BindingOrAssignmentElement'],
    },
    Block: {
        base: ['Statement'],
        getChildren: (node) => [node.statements],
        union: ['FunctionBody', 'BlockLike'],
    },
    BooleanKeyword: {
        base: ['TypeNode'],
        union: ['KeywordTypeNode'],
    },
    BreakKeyword: {
        base: ['Node'],
    },
    BreakStatement: {
        base: ['Statement'],
        getChildren: (node) => [node.label],
        union: ['BreakOrContinueStatement'],
    },
    Bundle: {
        base: ['Node'],
        getChildren: (node) => [node.sourceFiles],
    },
    CallExpression: {
        base: ['LeftHandSideExpression', 'Declaration'],
        getChildren: (node) => [
            node.expression,
            node.typeArguments,
            node.arguments,
        ],
        union: ['CallLikeExpression'],
    },
    CallSignature: {
        base: ['SignatureDeclarationBase', 'TypeElement'],
        union: ['HasJSDoc', 'SignatureDeclaration', 'FunctionLike'],
    },
    CaretEqualsToken: {
        base: ['Node'],
        union: ['CompoundAssignmentOperator'],
    },
    CaretToken: {
        base: ['Node'],
        union: ['BitwiseOperator'],
    },
    CaseBlock: {
        base: ['Node'],
        getChildren: (node) => [node.clauses],
    },
    CaseClause: {
        base: ['Node'],
        getChildren: (node) => [node.expression, node.statements],
        union: ['CaseOrDefaultClause'],
    },
    CaseKeyword: {
        base: ['Node'],
    },
    CatchClause: {
        base: ['Node'],
        getChildren: (node) => [
            node.variableDeclaration,
            node.block,
        ],
    },
    CatchKeyword: {
        base: ['Node'],
    },
    ClassDeclaration: {
        base: ['ClassLikeDeclarationBase', 'DeclarationStatement'],
        getChildren: (node) => [node.name],
        union: ['ClassLikeDeclaration'],
    },
    ClassExpression: {
        base: ['ClassLikeDeclarationBase', 'PrimaryExpression'],
        union: ['ClassLikeDeclaration'],
    },
    ClassKeyword: {
        base: ['Node'],
    },
    CloseBraceToken: {
        base: ['Node'],
    },
    CloseBracketToken: {
        base: ['Node'],
    },
    CloseParenToken: {
        base: ['Node'],
    },
    ColonToken: {
        base: ['Node'],
    },
    CommaListExpression: {
        base: ['Expression'],
        getChildren: (node) => [node.elements],
    },
    CommaToken: {
        base: ['Node'],
        union: ['BinaryOperator'],
    },
    ComputedPropertyName: {
        base: ['Node'],
        getChildren: (node) => [node.expression],
        union: ['PropertyName', 'DeclarationName'],
    },
    ConditionalExpression: {
        base: ['Expression'],
        getChildren: (node) => [
            node.condition,
            node.questionToken,
            node.whenTrue,
            node.colonToken,
            node.whenFalse,
        ],
    },
    ConflictMarkerTrivia: {
        base: ['Node'],
    },
    ConstKeyword: {
        base: ['Node'],
        union: ['Modifier'],
    },
    Constructor: {
        base: ['FunctionLikeDeclarationBase', 'ClassElement', 'JSDocContainer'],
        getChildren: (node) => [node.body],
        union: ['HasJSDoc', 'SignatureDeclaration', 'FunctionLikeDeclaration'],
    },
    ConstructorKeyword: {
        base: ['Node'],
    },
    ConstructorType: {
        base: ['TypeNode', 'SignatureDeclarationBase'],
        union: [
            'HasJSDoc',
            'SignatureDeclaration',
            'FunctionLike',
            'FunctionOrConstructorTypeNode',
        ],
    },
    ConstructSignature: {
        base: ['SignatureDeclarationBase', 'TypeElement'],
        union: ['HasJSDoc', 'SignatureDeclaration', 'FunctionLike'],
    },
    ContinueKeyword: {
        base: ['Node'],
    },
    ContinueStatement: {
        base: ['Statement'],
        getChildren: (node) => [node.label],
        union: ['BreakOrContinueStatement'],
    },
    Count: {
        base: ['Node'],
    },
    DebuggerKeyword: {
        base: ['Node'],
    },
    DebuggerStatement: {
        base: ['Statement'],
    },
    DeclareKeyword: {
        base: ['Node'],
        union: ['Modifier'],
    },
    Decorator: {
        base: ['Node'],
        getChildren: (node) => [node.expression],
        union: ['CallLikeExpression'],
    },
    DefaultClause: {
        base: ['Node'],
        getChildren: (node) => [node.statements],
        union: ['CaseOrDefaultClause'],
    },
    DefaultKeyword: {
        base: ['Node'],
        union: ['Modifier'],
    },
    DeleteExpression: {
        base: ['UnaryExpression'],
        getChildren: (node) => [node.expression],
    },
    DeleteKeyword: {
        base: ['Node'],
    },
    DoKeyword: {
        base: ['Node'],
    },
    DoStatement: {
        base: ['IterationStatement'],
        getChildren: (node) => [node.expression],
    },
    DotDotDotToken: {
        base: ['Node'],
    },
    DotToken: {
        base: ['Node'],
    },
    ElementAccessExpression: {
        base: ['MemberExpression'],
        getChildren: (node) => [
            node.expression,
            node.argumentExpression,
        ],
        union: ['BindingOrAssignmentElement'],
    },
    ElseKeyword: {
        base: ['Node'],
    },
    EmptyStatement: {
        base: ['Statement'],
    },
    EndOfDeclarationMarker: {
        base: ['Node'],
    },
    EndOfFileToken: {
        base: ['JSDocContainer'],
        union: ['HasJSDoc'],
    },
    EnumDeclaration: {
        base: ['DeclarationStatement', 'JSDocContainer'],
        getChildren: (node) => [node.name, node.members],
        union: ['HasJSDoc'],
    },
    EnumKeyword: {
        base: ['Node'],
    },
    EnumMember: {
        base: ['NamedDeclaration', 'JSDocContainer'],
        getChildren: (node) => [node.name, node.initializer],
        union: ['HasJSDoc'],
    },
    EqualsEqualsEqualsToken: {
        base: ['Node'],
        union: ['EqualityOperator'],
    },
    EqualsEqualsToken: {
        base: ['Node'],
        union: ['EqualityOperator'],
    },
    EqualsGreaterThanToken: {
        base: ['Node'],
    },
    EqualsToken: {
        base: ['Node'],
    },
    ExclamationEqualsEqualsToken: {
        base: ['Node'],
        union: ['EqualityOperator'],
    },
    ExclamationEqualsToken: {
        base: ['Node'],
        union: ['EqualityOperator'],
    },
    ExclamationToken: {
        base: ['Node'],
        union: ['PrefixUnaryOperator'],
    },
    ExportAssignment: {
        base: ['DeclarationStatement'],
        getChildren: (node) => [node.expression],
    },
    ExportDeclaration: {
        base: ['DeclarationStatement'],
        getChildren: (node) => [
            node.exportClause,
            node.moduleSpecifier,
        ],
    },
    ExportKeyword: {
        base: ['Node'],
        union: ['Modifier'],
    },
    ExportSpecifier: {
        base: ['NamedDeclaration'],
        getChildren: (node) => [node.propertyName, node.name],
        union: ['ImportOrExportSpecifier'],
    },
    ExpressionStatement: {
        base: ['Statement', 'JSDocContainer'],
        getChildren: (node) => [node.expression],
        union: ['HasJSDoc'],
    },
    ExpressionWithTypeArguments: {
        base: ['TypeNode'],
        getChildren: (node) => [
            node.expression,
            node.typeArguments,
        ],
        union: ['TypeReferenceType'],
    },
    ExtendsKeyword: {
        base: ['Node'],
    },
    ExternalModuleReference: {
        base: ['Node'],
        getChildren: (node) => [node.expression],
        union: ['ModuleReference'],
    },
    FalseKeyword: {
        base: ['PrimaryExpression', 'TypeNode'],
        union: ['BooleanLiteral'],
    },
    FinallyKeyword: {
        base: ['Node'],
    },
    ForInStatement: {
        base: ['IterationStatement'],
        getChildren: (node) => [
            node.initializer,
            node.expression,
        ],
        union: ['ForInOrOfStatement'],
    },
    ForKeyword: {
        base: ['Node'],
    },
    ForOfStatement: {
        base: ['IterationStatement'],
        getChildren: (node) => [
            node.awaitModifier,
            node.initializer,
            node.expression,
        ],
        union: ['ForInOrOfStatement'],
    },
    ForStatement: {
        base: ['IterationStatement'],
        getChildren: (node) => [
            node.initializer,
            node.condition,
            node.incrementor,
        ],
    },
    FromKeyword: {
        base: ['Node'],
    },
    FunctionDeclaration: {
        base: ['FunctionLikeDeclarationBase', 'DeclarationStatement'],
        getChildren: (node) => [node.name, node.body],
        union: ['HasJSDoc', 'SignatureDeclaration', 'FunctionLikeDeclaration'],
    },
    FunctionExpression: {
        base: [
            'PrimaryExpression',
            'FunctionLikeDeclarationBase',
            'JSDocContainer',
        ],
        getChildren: (node) => [node.name, node.body],
        union: ['HasJSDoc', 'SignatureDeclaration', 'FunctionLikeDeclaration'],
    },
    FunctionKeyword: {
        base: ['Node'],
    },
    FunctionType: {
        base: ['TypeNode', 'SignatureDeclarationBase'],
        union: [
            'HasJSDoc',
            'SignatureDeclaration',
            'FunctionLike',
            'FunctionOrConstructorTypeNode',
        ],
    },
    GetAccessor: {
        base: [
            'FunctionLikeDeclarationBase',
            'ClassElement',
            'ObjectLiteralElement',
            'JSDocContainer',
        ],
        getChildren: (node) => [node.name, node.body],
        union: ['FunctionLikeDeclaration', 'AccessorDeclaration'],
    },
    GetKeyword: {
        base: ['Node'],
    },
    GlobalKeyword: {
        base: ['Node'],
    },
    GreaterThanEqualsToken: {
        base: ['Node'],
        union: ['RelationalOperator'],
    },
    GreaterThanGreaterThanEqualsToken: {
        base: ['Node'],
        union: ['CompoundAssignmentOperator'],
    },
    GreaterThanGreaterThanGreaterThanEqualsToken: {
        base: ['Node'],
        union: ['CompoundAssignmentOperator'],
    },
    GreaterThanGreaterThanGreaterThanToken: {
        base: ['Node'],
        union: ['ShiftOperator'],
    },
    GreaterThanGreaterThanToken: {
        base: ['Node'],
        union: ['ShiftOperator'],
    },
    GreaterThanToken: {
        base: ['Node'],
        union: ['RelationalOperator'],
    },
    HeritageClause: {
        base: ['Node'],
        getChildren: (node) => [node.types],
    },
    Identifier: {
        base: ['PrimaryExpression'],
        union: [
            'EntityName',
            'PropertyName',
            'DeclarationName',
            'BindingName',
            'BindingOrAssignmentElement',
            'EntityNameExpression',
            'ModuleName',
            'JSDocNamespaceBody',
        ],
    },
    IfKeyword: {
        base: ['Node'],
    },
    IfStatement: {
        base: ['Statement'],
        getChildren: (node) => [
            node.expression,
            node.thenStatement,
            node.elseStatement,
        ],
    },
    ImplementsKeyword: {
        base: ['Node'],
    },
    ImportClause: {
        base: ['NamedDeclaration'],
        getChildren: (node) => [node.name, node.namedBindings],
    },
    ImportDeclaration: {
        base: ['Statement'],
        getChildren: (node) => [
            node.importClause,
            node.moduleSpecifier,
        ],
    },
    ImportEqualsDeclaration: {
        base: ['DeclarationStatement', 'JSDocContainer'],
        getChildren: (node) => [
            node.name,
            node.moduleReference,
        ],
        union: ['HasJSDoc'],
    },
    ImportKeyword: {
        base: ['PrimaryExpression'],
    },
    ImportSpecifier: {
        base: ['NamedDeclaration'],
        getChildren: (node) => [node.propertyName, node.name],
        union: ['ImportOrExportSpecifier'],
    },
    IndexedAccessType: {
        base: ['TypeNode'],
        getChildren: (node) => [
            node.objectType,
            node.indexType,
        ],
    },
    IndexSignature: {
        base: ['SignatureDeclarationBase', 'ClassElement', 'TypeElement'],
        union: ['HasJSDoc', 'SignatureDeclaration', 'FunctionLike'],
    },
    InKeyword: {
        base: ['Node'],
        union: ['RelationalOperator'],
    },
    InstanceOfKeyword: {
        base: ['Node'],
        union: ['RelationalOperator'],
    },
    InterfaceDeclaration: {
        base: ['DeclarationStatement', 'JSDocContainer'],
        getChildren: (node) => [
            node.name,
            node.typeParameters,
            node.heritageClauses,
            node.members,
        ],
        union: ['HasJSDoc', 'DeclarationWithTypeParameters'],
    },
    InterfaceKeyword: {
        base: ['Node'],
    },
    IntersectionType: {
        base: ['TypeNode'],
        getChildren: (node) => [node.types],
        union: ['UnionOrIntersectionTypeNode'],
    },
    IsKeyword: {
        base: ['Node'],
    },
    JSDocAllType: {
        base: ['JSDocType'],
    },
    JSDocAugmentsTag: {
        base: ['JSDocTag'],
        getChildren: (node) => [node.class],
    },
    JSDocClassTag: {
        base: ['JSDocTag'],
    },
    JSDocComment: {
        base: ['Node'],
        getChildren: (node) => [node.tags],
    },
    JSDocFunctionType: {
        base: ['JSDocType', 'SignatureDeclarationBase'],
        union: ['HasJSDoc', 'SignatureDeclaration'],
    },
    JSDocNonNullableType: {
        base: ['JSDocType'],
        getChildren: (node) => [node.type],
        union: ['JSDocTypeReferencingNode'],
    },
    JSDocNullableType: {
        base: ['JSDocType'],
        getChildren: (node) => [node.type],
        union: ['JSDocTypeReferencingNode'],
    },
    JSDocOptionalType: {
        base: ['JSDocType'],
        getChildren: (node) => [node.type],
        union: ['JSDocTypeReferencingNode'],
    },
    JSDocParameterTag: {
        base: ['JSDocPropertyLikeTag'],
    },
    JSDocPropertyTag: {
        base: ['JSDocPropertyLikeTag'],
    },
    JSDocReturnTag: {
        base: ['JSDocTag'],
        getChildren: (node) => [node.typeExpression],
    },
    JSDocTemplateTag: {
        base: ['JSDocTag'],
        getChildren: (node) => [node.typeParameters],
        union: ['DeclarationWithTypeParameters'],
    },
    JSDocTypedefTag: {
        base: ['JSDocTag', 'NamedDeclaration'],
        getChildren: (node) => [
            node.fullName,
            node.name,
            node.typeExpression,
        ],
    },
    JSDocTypeExpression: {
        base: ['TypeNode'],
        getChildren: (node) => [node.type],
    },
    JSDocTypeLiteral: {
        base: ['JSDocType'],
        getChildren: (node) => [node.jsDocPropertyTags],
    },
    JSDocTypeTag: {
        base: ['JSDocTag'],
        getChildren: (node) => [node.typeExpression],
    },
    JSDocUnknownType: {
        base: ['JSDocType'],
    },
    JSDocVariadicType: {
        base: ['JSDocType'],
        getChildren: (node) => [node.type],
        union: ['JSDocTypeReferencingNode'],
    },
    JsxAttribute: {
        base: ['ObjectLiteralElement'],
        getChildren: (node) => [node.name, node.initializer],
        union: ['JsxAttributeLike'],
    },
    JsxAttributes: {
        base: ['ObjectLiteralExpression'],
    },
    JsxClosingElement: {
        base: ['Node'],
        getChildren: (node) => [node.tagName],
    },
    JsxElement: {
        base: ['PrimaryExpression'],
        getChildren: (node) => [
            node.openingElement,
            node.children,
            node.closingElement,
        ],
        union: ['JsxChild'],
    },
    JsxExpression: {
        base: ['Expression'],
        getChildren: (node) => [
            node.dotDotDotToken,
            node.expression,
        ],
        union: ['JsxChild'],
    },
    JsxOpeningElement: {
        base: ['Expression'],
        getChildren: (node) => [
            node.tagName,
            node.attributes,
        ],
        union: ['JsxOpeningLikeElement'],
    },
    JsxSelfClosingElement: {
        base: ['PrimaryExpression'],
        getChildren: (node) => [
            node.tagName,
            node.attributes,
        ],
        union: ['JsxChild', 'JsxOpeningLikeElement'],
    },
    JsxSpreadAttribute: {
        base: ['ObjectLiteralElement'],
        getChildren: (node) => [node.expression],
        union: ['JsxAttributeLike'],
    },
    JsxText: {
        base: ['Node'],
        union: ['JsxChild'],
    },
    JsxTextAllWhiteSpaces: {
        base: ['Node'],
    },
    KeyOfKeyword: {
        base: ['Node'],
    },
    LabeledStatement: {
        base: ['Statement', 'JSDocContainer'],
        getChildren: (node) => [node.label, node.statement],
        union: ['HasJSDoc'],
    },
    LessThanEqualsToken: {
        base: ['Node'],
        union: ['RelationalOperator'],
    },
    LessThanLessThanEqualsToken: {
        base: ['Node'],
        union: ['CompoundAssignmentOperator'],
    },
    LessThanLessThanToken: {
        base: ['Node'],
        union: ['ShiftOperator'],
    },
    LessThanSlashToken: {
        base: ['Node'],
    },
    LessThanToken: {
        base: ['Node'],
        union: ['RelationalOperator'],
    },
    LetKeyword: {
        base: ['Node'],
    },
    LiteralType: {
        base: ['TypeNode'],
        getChildren: (node) => [node.literal],
    },
    MappedType: {
        base: ['TypeNode', 'Declaration'],
        getChildren: (node) => [
            node.readonlyToken,
            node.typeParameter,
            node.questionToken,
            node.type,
        ],
    },
    MergeDeclarationMarker: {
        base: ['Node'],
    },
    MetaProperty: {
        base: ['PrimaryExpression'],
        getChildren: (node) => [node.name],
    },
    MethodDeclaration: {
        base: [
            'FunctionLikeDeclarationBase',
            'ClassElement',
            'ObjectLiteralElement',
            'JSDocContainer',
        ],
        getChildren: (node) => [node.name, node.body],
        union: [
            'HasJSDoc',
            'SignatureDeclaration',
            'ObjectLiteralElementLike',
            'FunctionLikeDeclaration',
        ],
    },
    MethodSignature: {
        base: ['SignatureDeclarationBase', 'TypeElement'],
        getChildren: (node) => [node.name],
        union: ['HasJSDoc', 'SignatureDeclaration', 'FunctionLike'],
    },
    MinusEqualsToken: {
        base: ['Node'],
        union: ['CompoundAssignmentOperator'],
    },
    MinusMinusToken: {
        base: ['Node'],
        union: ['PrefixUnaryOperator', 'PostfixUnaryOperator'],
    },
    MinusToken: {
        base: ['Node'],
        union: ['PrefixUnaryOperator'],
    },
    MissingDeclaration: {
        base: [
            'DeclarationStatement',
            'ClassElement',
            'ObjectLiteralElement',
            'TypeElement',
        ],
        getChildren: (node) => [node.name],
    },
    ModuleBlock: {
        base: ['Statement'],
        getChildren: (node) => [node.statements],
        union: ['BlockLike', 'NamespaceBody'],
    },
    ModuleDeclaration: {
        base: ['DeclarationStatement', 'JSDocContainer'],
        getChildren: (node) => [node.name, node.body],
        union: ['HasJSDoc'],
    },
    ModuleKeyword: {
        base: ['Node'],
    },
    MultiLineCommentTrivia: {
        base: ['Node'],
        union: ['CommentKind'],
    },
    NamedExports: {
        base: ['Node'],
        getChildren: (node) => [node.elements],
        union: ['NamedImportsOrExports'],
    },
    NamedImports: {
        base: ['Node'],
        getChildren: (node) => [node.elements],
        union: ['NamedImportsOrExports'],
    },
    NamespaceExportDeclaration: {
        base: ['DeclarationStatement'],
        getChildren: (node) => [node.name],
    },
    NamespaceImport: {
        base: ['NamedDeclaration'],
        getChildren: (node) => [node.name],
        union: ['NamedImportBindings'],
    },
    NamespaceKeyword: {
        base: ['Node'],
    },
    NeverKeyword: {
        base: ['TypeNode'],
        union: ['KeywordTypeNode'],
    },
    NewExpression: {
        base: ['PrimaryExpression', 'Declaration'],
        getChildren: (node) => [
            node.expression,
            node.typeArguments,
            node.arguments,
        ],
        union: ['CallLikeExpression'],
    },
    NewKeyword: {
        base: ['Node'],
    },
    NewLineTrivia: {
        base: ['Node'],
    },
    NonNullExpression: {
        base: ['LeftHandSideExpression'],
        getChildren: (node) => [node.expression],
    },
    NoSubstitutionTemplateLiteral: {
        base: ['LiteralExpression'],
        union: ['TemplateLiteral'],
    },
    NotEmittedStatement: {
        base: ['Statement'],
    },
    NullKeyword: {
        base: ['PrimaryExpression', 'TypeNode'],
        union: ['KeywordTypeNode'],
    },
    NumberKeyword: {
        base: ['TypeNode'],
        union: ['KeywordTypeNode'],
    },
    NumericLiteral: {
        base: ['LiteralExpression'],
        union: ['PropertyName', 'DeclarationName'],
    },
    ObjectBindingPattern: {
        base: ['Node'],
        getChildren: (node) => [node.elements],
        union: ['BindingPattern', 'ObjectBindingOrAssignmentPattern'],
    },
    ObjectKeyword: {
        base: ['TypeNode'],
        union: ['KeywordTypeNode'],
    },
    ObjectLiteralExpression: {
        base: ['ObjectLiteralExpressionBase'],
        union: [
            'BindingOrAssignmentElement',
            'ObjectBindingOrAssignmentPattern',
            'AssignmentPattern',
            'DestructuringPattern',
        ],
    },
    OfKeyword: {
        base: ['Node'],
    },
    OmittedExpression: {
        base: ['Expression'],
        union: ['ArrayBindingElement', 'BindingOrAssignmentElement'],
    },
    OpenBraceToken: {
        base: ['Node'],
    },
    OpenBracketToken: {
        base: ['Node'],
    },
    OpenParenToken: {
        base: ['Node'],
    },
    PackageKeyword: {
        base: ['Node'],
    },
    Parameter: {
        base: ['NamedDeclaration', 'JSDocContainer'],
        getChildren: (node) => [
            node.dotDotDotToken,
            node.name,
            node.questionToken,
            node.type,
            node.initializer,
        ],
        union: ['BindingOrAssignmentElement'],
    },
    ParenthesizedExpression: {
        base: ['PrimaryExpression', 'JSDocContainer'],
        getChildren: (node) => [node.expression],
        union: ['HasJSDoc', 'EntityNameExpression'],
    },
    ParenthesizedType: {
        base: ['TypeNode'],
        getChildren: (node) => [node.type],
    },
    PartiallyEmittedExpression: {
        base: ['LeftHandSideExpression'],
        getChildren: (node) => [node.expression],
    },
    PercentEqualsToken: {
        base: ['Node'],
        union: ['CompoundAssignmentOperator'],
    },
    PercentToken: {
        base: ['Node'],
        union: ['MultiplicativeOperator'],
    },
    PlusEqualsToken: {
        base: ['Node'],
        union: ['CompoundAssignmentOperator'],
    },
    PlusPlusToken: {
        base: ['Node'],
        union: ['PrefixUnaryOperator', 'PostfixUnaryOperator'],
    },
    PlusToken: {
        base: ['Node'],
        union: ['PrefixUnaryOperator', 'AdditiveOperator'],
    },
    PostfixUnaryExpression: {
        base: ['UpdateExpression'],
        getChildren: (node) => [node.operand],
    },
    PrefixUnaryExpression: {
        base: ['UpdateExpression'],
        getChildren: (node) => [node.operand],
    },
    PrivateKeyword: {
        base: ['Node'],
        union: ['Modifier'],
    },
    PropertyAccessExpression: {
        base: ['MemberExpression', 'NamedDeclaration'],
        getChildren: (node) => [
            node.expression,
            node.name,
        ],
        union: ['BindingOrAssignmentElement'],
    },
    PropertyAssignment: {
        base: ['ObjectLiteralElement', 'JSDocContainer'],
        getChildren: (node) => [
            node.name,
            node.questionToken,
            node.initializer,
        ],
        union: ['ObjectLiteralElementLike', 'BindingOrAssignmentElement'],
    },
    PropertyDeclaration: {
        base: ['ClassElement', 'JSDocContainer'],
        getChildren: (node) => [
            node.questionToken,
            node.name,
            node.type,
            node.initializer,
        ],
        union: ['HasJSDoc'],
    },
    PropertySignature: {
        base: ['TypeElement', 'JSDocContainer'],
        getChildren: (node) => [
            node.name,
            node.questionToken,
            node.type,
            node.initializer,
        ],
        union: ['HasJSDoc'],
    },
    ProtectedKeyword: {
        base: ['Node'],
        union: ['Modifier'],
    },
    PublicKeyword: {
        base: ['Node'],
        union: ['Modifier'],
    },
    QualifiedName: {
        base: ['Node'],
        getChildren: (node) => [node.left, node.right],
        union: ['EntityName'],
    },
    QuestionToken: {
        base: ['Node'],
    },
    ReadonlyKeyword: {
        base: ['Node'],
        union: ['Modifier'],
    },
    RegularExpressionLiteral: {
        base: ['LiteralExpression'],
    },
    RequireKeyword: {
        base: ['Node'],
    },
    ReturnKeyword: {
        base: ['Node'],
    },
    ReturnStatement: {
        base: ['Statement'],
        getChildren: (node) => [node.expression],
    },
    SemicolonClassElement: {
        base: ['ClassElement'],
    },
    SemicolonToken: {
        base: ['Node'],
    },
    SetAccessor: {
        base: [
            'FunctionLikeDeclarationBase',
            'ClassElement',
            'ObjectLiteralElement',
            'JSDocContainer',
        ],
        getChildren: (node) => [node.name, node.body],
        union: ['FunctionLikeDeclaration', 'AccessorDeclaration'],
    },
    SetKeyword: {
        base: ['Node'],
    },
    ShebangTrivia: {
        base: ['Node'],
    },
    ShorthandPropertyAssignment: {
        base: ['ObjectLiteralElement'],
        getChildren: (node) => [
            node.name,
            node.questionToken,
            node.equalsToken,
            node.objectAssignmentInitializer,
        ],
        union: [
            'HasJSDoc',
            'ObjectLiteralElementLike',
            'BindingOrAssignmentElement',
        ],
    },
    SingleLineCommentTrivia: {
        base: ['Node'],
        union: ['CommentKind'],
    },
    SlashEqualsToken: {
        base: ['Node'],
        union: ['CompoundAssignmentOperator'],
    },
    SlashToken: {
        base: ['Node'],
        union: ['MultiplicativeOperator'],
    },
    SourceFile: {
        base: ['Declaration'],
        getChildren: (node) => [
            node.statements,
            node.endOfFileToken,
        ],
        union: ['BlockLike'],
    },
    SpreadAssignment: {
        base: ['ObjectLiteralElement', 'JSDocContainer'],
        getChildren: (node) => [node.expression],
        union: [
            'HasJSDoc',
            'ObjectLiteralElementLike',
            'BindingOrAssignmentElement',
            'BindingOrAssignmentElementRestIndicator',
        ],
    },
    SpreadElement: {
        base: ['Expression'],
        getChildren: (node) => [node.expression],
        union: [
            'BindingOrAssignmentElement',
            'BindingOrAssignmentElementRestIndicator',
        ],
    },
    StaticKeyword: {
        base: ['Node'],
        union: ['Modifier'],
    },
    StringKeyword: {
        base: ['TypeNode'],
        union: ['KeywordTypeNode'],
    },
    StringLiteral: {
        base: ['LiteralExpression'],
        union: ['PropertyName', 'DeclarationName', 'ModuleName'],
    },
    SuperKeyword: {
        base: ['PrimaryExpression'],
    },
    SwitchKeyword: {
        base: ['Node'],
    },
    SwitchStatement: {
        base: ['Statement'],
        getChildren: (node) => [
            node.expression,
            node.caseBlock,
        ],
    },
    SymbolKeyword: {
        base: ['TypeNode'],
        union: ['KeywordTypeNode'],
    },
    SyntaxList: {
        base: ['Node'],
        getChildren: (node) => [node._children],
    },
    TaggedTemplateExpression: {
        base: ['MemberExpression'],
        getChildren: (node) => [
            node.tag,
            node.template,
        ],
        union: ['CallLikeExpression'],
    },
    TemplateExpression: {
        base: ['PrimaryExpression'],
        getChildren: (node) => [
            node.head,
            node.templateSpans,
        ],
        union: ['TemplateLiteral'],
    },
    TemplateHead: {
        base: ['LiteralLikeNode'],
    },
    TemplateMiddle: {
        base: ['LiteralLikeNode'],
    },
    TemplateSpan: {
        base: ['Node'],
        getChildren: (node) => [node.expression, node.literal],
    },
    TemplateTail: {
        base: ['LiteralLikeNode'],
    },
    ThisKeyword: {
        base: ['TypeNode'],
        union: ['KeywordTypeNode'],
    },
    ThisType: {
        base: ['TypeNode'],
    },
    ThrowKeyword: {
        base: ['Node'],
    },
    ThrowStatement: {
        base: ['Statement'],
        getChildren: (node) => [node.expression],
    },
    TildeToken: {
        base: ['Node'],
        union: ['PrefixUnaryOperator'],
    },
    TrueKeyword: {
        base: ['PrimaryExpression', 'TypeNode'],
        union: ['BooleanLiteral'],
    },
    TryKeyword: {
        base: ['Node'],
    },
    TryStatement: {
        base: ['Statement'],
        getChildren: (node) => [
            node.tryBlock,
            node.catchClause,
            node.finallyBlock,
        ],
    },
    TupleType: {
        base: ['TypeNode'],
        getChildren: (node) => [node.elementTypes],
    },
    TypeAliasDeclaration: {
        base: ['DeclarationStatement', 'JSDocContainer'],
        getChildren: (node) => [
            node.name,
            node.typeParameters,
            node.type,
        ],
        union: ['HasJSDoc', 'DeclarationWithTypeParameters'],
    },
    TypeAssertionExpression: {
        base: ['UnaryExpression'],
        getChildren: (node) => [node.type, node.expression],
        union: ['AssertionExpression'],
    },
    TypeKeyword: {
        base: ['Node'],
    },
    TypeLiteral: {
        base: ['TypeNode', 'Declaration'],
        getChildren: (node) => [node.members],
    },
    TypeOfExpression: {
        base: ['UnaryExpression'],
        getChildren: (node) => [node.expression],
    },
    TypeOfKeyword: {
        base: ['Node'],
    },
    TypeOperator: {
        base: ['TypeNode'],
        getChildren: (node) => [node.type],
    },
    TypeParameter: {
        base: ['NamedDeclaration'],
        getChildren: (node) => [
            node.name,
            node.constraint,
            node.default,
            node.expression,
        ],
    },
    TypePredicate: {
        base: ['TypeNode'],
        getChildren: (node) => [
            node.parameterName,
            node.type,
        ],
    },
    TypeQuery: {
        base: ['TypeNode'],
        getChildren: (node) => [node.exprName],
    },
    TypeReference: {
        base: ['TypeNode'],
        getChildren: (node) => [
            node.typeName,
            node.typeArguments,
        ],
        union: ['TypeReferenceType'],
    },
    UndefinedKeyword: {
        base: ['TypeNode'],
        union: ['KeywordTypeNode'],
    },
    UnionType: {
        base: ['TypeNode'],
        getChildren: (node) => [node.types],
        union: ['UnionOrIntersectionTypeNode'],
    },
    Unknown: {
        base: ['Node'],
    },
    VariableDeclaration: {
        base: ['NamedDeclaration'],
        getChildren: (node) => [
            node.name,
            node.type,
            node.initializer,
        ],
        union: ['BindingOrAssignmentElement'],
    },
    VariableDeclarationList: {
        base: ['Node'],
        getChildren: (node) => [node.declarations],
        union: ['ForInitializer'],
    },
    VariableStatement: {
        base: ['Statement', 'JSDocContainer'],
        getChildren: (node) => [node.declarationList],
        union: ['HasJSDoc'],
    },
    VarKeyword: {
        base: ['Node'],
    },
    VoidExpression: {
        base: ['UnaryExpression'],
        getChildren: (node) => [node.expression],
    },
    VoidKeyword: {
        base: ['TypeNode'],
        union: ['KeywordTypeNode'],
    },
    WhileKeyword: {
        base: ['Node'],
    },
    WhileStatement: {
        base: ['IterationStatement'],
        getChildren: (node) => [node.expression],
    },
    WhitespaceTrivia: {
        base: ['Node'],
    },
    WithKeyword: {
        base: ['Node'],
    },
    WithStatement: {
        base: ['Statement'],
        getChildren: (node) => [node.expression, node.statement],
    },
    YieldExpression: {
        base: ['Expression'],
        getChildren: (node) => [
            node.asteriskToken,
            node.expression,
        ],
    },
    YieldKeyword: {
        base: ['Node'],
    },
};
exports.default = kinds;
//# sourceMappingURL=kinds.js.map