"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const unions = {
    AccessorDeclaration: {
        base: ['Node'],
        union: ['HasJSDoc', 'SignatureDeclaration', 'ObjectLiteralElementLike'],
    },
    AdditiveOperator: {
        base: ['Node'],
        union: ['AdditiveOperatorOrHigher'],
    },
    AdditiveOperatorOrHigher: {
        base: ['Node'],
        union: ['ShiftOperatorOrHigher'],
    },
    ArrayBindingElement: {
        base: ['Node'],
    },
    ArrayBindingOrAssignmentPattern: {
        base: ['Node'],
        union: ['BindingOrAssignmentPattern'],
    },
    AssertionExpression: {
        base: ['Node'],
    },
    AssignmentOperator: {
        base: ['Node'],
        union: ['AssignmentOperatorOrHigher'],
    },
    AssignmentOperatorOrHigher: {
        base: ['Node'],
        union: ['BinaryOperator'],
    },
    AssignmentPattern: {
        base: ['Node'],
    },
    BinaryOperator: {
        base: ['Node'],
    },
    BindingName: {
        base: ['Node'],
    },
    BindingOrAssignmentElement: {
        base: ['Node'],
    },
    BindingOrAssignmentElementRestIndicator: {
        base: ['Node'],
    },
    BindingOrAssignmentElementTarget: {
        base: ['Node'],
    },
    BindingOrAssignmentPattern: {
        base: ['Node'],
        union: ['BindingOrAssignmentElementTarget'],
    },
    BindingPattern: {
        base: ['Node'],
        union: ['DeclarationName', 'BindingName', 'DestructuringPattern'],
    },
    BitwiseOperator: {
        base: ['Node'],
        union: ['BitwiseOperatorOrHigher'],
    },
    BitwiseOperatorOrHigher: {
        base: ['Node'],
        union: ['LogicalOperatorOrHigher'],
    },
    BlockLike: {
        base: ['Node'],
    },
    BooleanLiteral: {
        base: ['Node'],
    },
    BreakOrContinueStatement: {
        base: ['Node'],
    },
    CallLikeExpression: {
        base: ['Node'],
    },
    CaseOrDefaultClause: {
        base: ['Node'],
        union: ['BlockLike'],
    },
    ClassLikeDeclaration: {
        base: ['Node'],
        union: ['HasJSDoc', 'DeclarationWithTypeParameters'],
    },
    CommentKind: {
        base: ['Node'],
    },
    CompoundAssignmentOperator: {
        base: ['Node'],
        union: ['AssignmentOperator'],
    },
    ConciseBody: {
        base: ['Node'],
    },
    DeclarationName: {
        base: ['Node'],
    },
    DeclarationWithTypeParameters: {
        base: ['Node'],
    },
    DestructuringPattern: {
        base: ['Node'],
    },
    EntityName: {
        base: ['Node'],
        union: ['EntityNameOrEntityNameExpression', 'ModuleReference'],
    },
    EntityNameExpression: {
        base: ['Node'],
    },
    EntityNameOrEntityNameExpression: {
        base: ['Node'],
    },
    EqualityOperator: {
        base: ['Node'],
        union: ['EqualityOperatorOrHigher'],
    },
    EqualityOperatorOrHigher: {
        base: ['Node'],
        union: ['BitwiseOperatorOrHigher'],
    },
    ExponentiationOperator: {
        base: ['Node'],
        union: ['MultiplicativeOperatorOrHigher'],
    },
    ForInitializer: {
        base: ['Node'],
    },
    ForInOrOfStatement: {
        base: ['Node'],
    },
    FunctionBody: {
        base: ['Node'],
        union: ['ConciseBody'],
    },
    FunctionLike: {
        base: ['Node'],
    },
    FunctionLikeDeclaration: {
        base: ['Node'],
        union: ['FunctionLike'],
    },
    FunctionOrConstructorTypeNode: {
        base: ['Node'],
    },
    HasJSDoc: {
        base: ['Node'],
    },
    ImportOrExportSpecifier: {
        base: ['Node'],
    },
    JSDocNamespaceBody: {
        base: ['Node'],
        union: ['ModuleBody'],
    },
    JSDocTypeReferencingNode: {
        base: ['Node'],
    },
    JsxAttributeLike: {
        base: ['Node'],
    },
    JsxChild: {
        base: ['Node'],
    },
    JsxOpeningLikeElement: {
        base: ['Node'],
        union: ['CallLikeExpression'],
    },
    KeywordTypeNode: {
        base: ['TypeNode'],
    },
    LogicalOperator: {
        base: ['Node'],
        union: ['LogicalOperatorOrHigher'],
    },
    LogicalOperatorOrHigher: {
        base: ['Node'],
        union: ['AssignmentOperatorOrHigher'],
    },
    Modifier: {
        base: ['Node'],
    },
    ModuleBody: {
        base: ['Node'],
    },
    ModuleName: {
        base: ['Node'],
    },
    ModuleReference: {
        base: ['Node'],
    },
    MultiplicativeOperator: {
        base: ['Node'],
        union: ['MultiplicativeOperatorOrHigher'],
    },
    MultiplicativeOperatorOrHigher: {
        base: ['Node'],
        union: ['AdditiveOperatorOrHigher'],
    },
    NamedImportBindings: {
        base: ['Node'],
    },
    NamedImportsOrExports: {
        base: ['Node'],
    },
    NamespaceBody: {
        base: ['Node'],
        union: ['ModuleBody'],
    },
    ObjectBindingOrAssignmentPattern: {
        base: ['Node'],
        union: ['BindingOrAssignmentPattern'],
    },
    ObjectLiteralElementLike: {
        base: ['Node'],
    },
    PostfixUnaryOperator: {
        base: ['Node'],
    },
    PrefixUnaryOperator: {
        base: ['Node'],
    },
    PropertyName: {
        base: ['Node'],
    },
    RelationalOperator: {
        base: ['Node'],
        union: ['RelationalOperatorOrHigher'],
    },
    RelationalOperatorOrHigher: {
        base: ['Node'],
        union: ['EqualityOperatorOrHigher'],
    },
    ShiftOperator: {
        base: ['Node'],
        union: ['ShiftOperatorOrHigher'],
    },
    ShiftOperatorOrHigher: {
        base: ['Node'],
        union: ['RelationalOperatorOrHigher'],
    },
    SignatureDeclaration: {
        base: ['Node'],
        union: ['DeclarationWithTypeParameters'],
    },
    TemplateLiteral: {
        base: ['Node'],
    },
    TypeReferenceType: {
        base: ['Node'],
    },
    UnionOrIntersectionTypeNode: {
        base: ['Node'],
    },
};
exports.default = unions;
//# sourceMappingURL=unions.js.map