"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts = require("typescript");
const types_1 = require("./types");
const utils_1 = require("./utils");
function visitNode(node, visitor, kind, iterateBase) {
    if (iterateBase) {
        for (const key of resolveUnion(kind)) {
            visitNode(node, visitor, key, false);
        }
    }
    const baseKinds = resolveBase(kind);
    if (iterateBase) {
        for (const key of baseKinds) {
            visitNode(node, visitor, key, false);
        }
    }
    const visitorFunc = visitor[kind] || visitor.Default;
    if (visitorFunc(node, kind) === false) {
        return;
    }
    if (iterateBase) {
        visitChildren(visitor, resolveChildren(node, baseKinds));
    }
}
exports.default = visitNode;
function resolveBase(kind) {
    const base = utils_1.defs[kind].base;
    return base
        .reduce((r, d) => r.concat(resolveBase(d)), [...base])
        .filter((o, i, a) => a.indexOf(o) === i);
}
function resolveUnion(kind) {
    const union = utils_1.defs[kind].union || [];
    return union
        .reduce((r, d) => r.concat(resolveUnion(d)), [...union])
        .filter((o, i, a) => a.indexOf(o) === i);
}
function resolveChildren(node, baseKinds) {
    return [...baseKinds, getKind(node)]
        .map(k => utils_1.defs[k].getChildren)
        .reduce((r, w) => (w ? r.concat(w(node)) : r), [])
        .filter(o => o !== undefined)
        .filter((o, i, a) => a.indexOf(o) === i);
}
function visitChildren(visitor, items) {
    for (const item of items) {
        if (isArray(item)) {
            for (const child of item) {
                visitNode(child, visitor, getKind(child), true);
            }
        }
        else {
            visitNode(item, visitor, getKind(item), true);
        }
    }
}
function isArray(node) {
    return Array.isArray(node);
}
function getKind(node) {
    let value = ts.SyntaxKind[node.kind];
    if (types_1.IgnoredKinds.hasOwnProperty(value)) {
        value = types_1.ignoredKindsMap[value];
    }
    return value;
}
//# sourceMappingURL=visitNode.js.map