import * as ts from 'typescript';

declare enum Unions {
  AccessorDeclaration,
  AdditiveOperator,
  AdditiveOperatorOrHigher,
  ArrayBindingElement,
  ArrayBindingOrAssignmentPattern,
  AssertionExpression,
  AssignmentOperator,
  AssignmentOperatorOrHigher,
  AssignmentPattern,
  BinaryOperator,
  BindingName,
  BindingOrAssignmentElement,
  BindingOrAssignmentElementRestIndicator,
  BindingOrAssignmentElementTarget,
  BindingOrAssignmentPattern,
  BindingPattern,
  BitwiseOperator,
  BitwiseOperatorOrHigher,
  BlockLike,
  BooleanLiteral,
  BreakOrContinueStatement,
  CallLikeExpression,
  CaseOrDefaultClause,
  ClassLikeDeclaration,
  CommentKind,
  CompoundAssignmentOperator,
  ConciseBody,
  DeclarationName,
  DeclarationWithTypeParameters,
  DestructuringPattern,
  EntityName,
  EntityNameExpression,
  EntityNameOrEntityNameExpression,
  EqualityOperator,
  EqualityOperatorOrHigher,
  ExponentiationOperator,
  ForInitializer,
  ForInOrOfStatement,
  FunctionBody,
  FunctionLike,
  FunctionLikeDeclaration,
  FunctionOrConstructorTypeNode,
  HasJSDoc,
  ImportOrExportSpecifier,
  JSDocNamespaceBody,
  JSDocTypeReferencingNode,
  JsxAttributeLike,
  JsxChild,
  JsxOpeningLikeElement,
  KeywordTypeNode,
  LogicalOperator,
  LogicalOperatorOrHigher,
  Modifier,
  ModuleBody,
  ModuleName,
  ModuleReference,
  MultiplicativeOperator,
  MultiplicativeOperatorOrHigher,
  NamedImportBindings,
  NamedImportsOrExports,
  NamespaceBody,
  ObjectBindingOrAssignmentPattern,
  ObjectLiteralElementLike,
  PostfixUnaryOperator,
  PrefixUnaryOperator,
  PropertyName,
  RelationalOperator,
  RelationalOperatorOrHigher,
  ShiftOperator,
  ShiftOperatorOrHigher,
  SignatureDeclaration,
  TemplateLiteral,
  TypeReferenceType,
  UnionOrIntersectionTypeNode,
}

declare enum Base {
  ClassElement,
  ClassLikeDeclarationBase,
  Declaration,
  DeclarationStatement,
  Expression,
  FunctionLikeDeclarationBase,
  IterationStatement,
  JSDocContainer,
  JSDocPropertyLikeTag,
  JSDocTag,
  JSDocType,
  LeftHandSideExpression,
  LiteralExpression,
  LiteralLikeNode,
  MemberExpression,
  NamedDeclaration,
  Node,
  ObjectLiteralElement,
  ObjectLiteralExpression,
  ObjectLiteralExpressionBase,
  PrimaryExpression,
  SignatureDeclarationBase,
  Statement,
  TypeElement,
  TypeNode,
  UnaryExpression,
  UpdateExpression,
}

declare type DefKind =
  | keyof typeof ts.SyntaxKind
  | keyof typeof Unions
  | keyof typeof Base;

/**
 * Signature of visitor function.
 * Return false to skip visiting child nodes.
 */
export declare type VisitorFunc<TNode extends ts.Node> = (
  node: TNode,
  kind: DefKind,
) => void | boolean;

/**
 * Visitor type. If method is not declared then `Default` method is called.
 */
export declare type Visitor = { [kind in DefKind]?: VisitorFunc<any> } & {
  Default: VisitorFunc<ts.Node>;
};


/**
 * Visits AST tree from given node and calls corresponding methods in visitor.
 */
export declare function visit(node: ts.Node, visitor: Visitor): void;
